#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 17.08.2017 17:28
:Licence GNUv3
Part of grammpy

"""

from .PrettyApiGrammar import PrettyApiGrammar
from ..Constants import EPSILON

class RulesRemovingGrammar(PrettyApiGrammar):
    def __init__(self,
                 terminals=None,
                 nonterminals=None,
                 rules=None,
                 start_symbol=None):
        self._symbs_of_rules = dict({EPSILON: set()})
        super().__init__(terminals, nonterminals, rules, start_symbol)

    def add_term(self, term):
        add = super().add_term(term)
        for t in add: self._symbs_of_rules[t.s] = set()
        return add

    def remove_term(self, term=None):
        rem = super().remove_term(term)
        for term in rem:
            self.remove_rule(list(self._symbs_of_rules[term.s]), _validate=False)
            del self._symbs_of_rules[term.s]
        return rem

    def add_nonterm(self, nonterms):
        add = super().add_nonterm(nonterms)
        for n in add: self._symbs_of_rules[n] = set()
        return add

    def remove_nonterm(self, nonterms=None):
        rem = super().remove_nonterm(nonterms)
        for nonterm in rem:
            self.remove_rule(list(self._symbs_of_rules[nonterm]), _validate=False)
            del self._symbs_of_rules[nonterm]
        if self.start_get() in rem: self.start_set(None)
        return rem

    def add_rule(self, rules):
        add = super().add_rule(rules)
        for inst in add:
            for rule in inst.rules:
                for symb in rule[0]:
                    self._symbs_of_rules[symb].add(inst)
                for symb in rule[1]:
                    self._symbs_of_rules[symb].add(inst)
        return add

    def _remove_if_exists(self, key, val):
        if val in self._symbs_of_rules[key]:
            self._symbs_of_rules[key].remove(val)

    def remove_rule(self, rules=None, *, _validate=True):
        rem = super().remove_rule(rules, _validate=_validate)
        for inst in rem:
            for rule in inst.rules:
                for side in rule:
                    for symb in side:
                        self._remove_if_exists(symb, inst)
        return rem



#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 16.08.2017 19:33
:Licence GNUv3
Part of grammpy

"""

from .IsMethodsRuleExtension import IsMethodsRuleExtension


class ActivtableRule(IsMethodsRuleExtension):
    _active = True



#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 15.08.2017 14:40
:Licence GNUv3
Part of grammpy

"""
import inspect

from ..Rules import Rule
from .StringGrammar import StringGrammar
from ..HashContainer import HashContainer


class MultipleRulesGrammar(StringGrammar):
    def __init__(self,
                 terminals=None,
                 nonterminals=None,
                 rules=None,
                 start_symbol=None):
        self._count = 0
        super().__init__(terminals, nonterminals, rules, start_symbol)

    def _create_class(self, rule):
        name = 'SplitRules' + str(self._count)
        self._count += 1
        return type(name,
                    (Rule,),
                    {"rule": rule})

    def _transform_rules(self, rules, *, _validate=True):
        rules = HashContainer.to_iterable(rules)
        r = []
        for i in rules:
            if not inspect.isclass(i) or not issubclass(i, Rule):
                r.append(i)
            elif (i.is_valid(self) and i.count() > 1) or (not _validate and i.count() > 1):
                for rule in i.rules:
                    r.append(self._create_class(rule))
            else:
                r.append(i)
        return r

    def get_rule(self, rules=None):
        if rules is None:
            return super().get_rule()
        results = super().get_rule(self._transform_rules(rules))
        if not HashContainer.is_iterable(rules) and rules.count() == 1:
            return results[0]
        return results

    def have_rule(self, rules):
        return super().have_rule(self._transform_rules(rules))

    def remove_rule(self, rules=None, *, _validate=True):
        if rules is None:
            return super().remove_rule(_validate=_validate)
        return super().remove_rule(self._transform_rules(rules, _validate=_validate), _validate=_validate)

    def add_rule(self, rules):
        return super().add_rule(self._transform_rules(rules))
